package cd.casic.plugin.extension;

import org.apache.commons.lang3.StringUtils;
import org.springframework.data.domain.Sort;

import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import static org.springframework.data.domain.Sort.Order.asc;
import static org.springframework.data.domain.Sort.Order.desc;

/**
 * @Classname ExtensionUtil
 * @Description TODO
 * @Date 2025/5/8 15:12
 * @Created by mianbin
 */
public enum ExtensionUtil {
    ;

    public static boolean isDeleted(ExtensionOperator extension) {
        return extension.getMetadata() != null
                && extension.getMetadata().getDeletionTimestamp() != null;
    }

    public static boolean addFinalizers(MetadataOperator metadata, Set<String> finalizers) {
        var modifiableFinalizers = new HashSet<>(
                metadata.getFinalizers() == null ? Collections.emptySet() : metadata.getFinalizers());
        var added = modifiableFinalizers.addAll(finalizers);
        if (added) {
            metadata.setFinalizers(modifiableFinalizers);
        }
        return added;
    }

    public static boolean removeFinalizers(MetadataOperator metadata, Set<String> finalizers) {
        if (metadata.getFinalizers() == null) {
            return false;
        }
        var existingFinalizers = new HashSet<>(metadata.getFinalizers());
        var removed = existingFinalizers.removeAll(finalizers);
        if (removed) {
            metadata.setFinalizers(existingFinalizers);
        }
        return removed;
    }

    /**
     * Query for not deleting.
     *
     * @return Query
     */
    public static boolean notDeleting() {
        return StringUtils.isNotBlank("metadata.deletionTimestamp");
    }

    /**
     * Default sort by creation timestamp desc and name asc.
     *
     * @return Sort
     */
    public static Sort defaultSort() {
        return Sort.by(
                desc("metadata.creationTimestamp"),
                asc("metadata.name")
        );
    }
}
