package cd.casic.module.process.support.postprocess.service.impl;

import cd.casic.ci.commons.bean.engine.job.Tasks;
import cd.casic.ci.commons.bean.engine.postprocess.Postprocess;
import cd.casic.ci.commons.bean.engine.postprocess.PostprocessInstance;
import cd.casic.module.process.engine.task.TasksExecService;
import cd.casic.module.process.engine.task.TasksService;
import cd.casic.module.process.support.postprocess.service.PostprocessExecService;
import cd.casic.module.process.support.postprocess.service.PostprocessInstanceService;
import cd.casic.module.process.support.postprocess.service.PostprocessService;
import cd.casic.module.process.support.util.PipelineUtilService;
import cd.casic.module.process.support.util.util.PipelineFinal;
import jakarta.annotation.Resource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @ClassName PostprocessExecServiceImpl
 * @Author hopeli
 * @Date 2025/4/28 15:39
 * @Version 1.0
 */

@Service
public class PostprocessExecServiceImpl implements PostprocessExecService {
    @Resource
    PostprocessService postprocessService;

    @Resource
    TasksExecService tasksExecService;

    @Resource
    PostprocessInstanceService postInstanceService;

    @Resource
    PipelineUtilService utilService;

    @Resource
    TasksService tasksService;

    private static final Logger logger = LoggerFactory.getLogger(PostprocessExecServiceImpl.class);


    //后置任务id与后置任务实例id之间的关系
    public static Map<String , String> postIdOrPostInstanceId = new HashMap<>();

    //后置实例id与后置任务实例之间的关系
    public static Map<String , PostprocessInstance> postInstanceIdOrPostInstance = new HashMap<>();


    private final static Map<String , List<String>> pipelineIdOrPostInstanceId = new HashMap<>();

    @Override
    public List<Postprocess> createPipelinePostInstance(String pipelineId, String instanceId){
        String fileAddress = utilService.findPipelineDefaultAddress(pipelineId,2) + instanceId;
        List<Postprocess> postprocessList = postprocessService.findAllPipelinePostTask(pipelineId);
        if (postprocessList.isEmpty()){
            return Collections.emptyList();
        }
        for (Postprocess postprocess : postprocessList) {

            PostprocessInstance postInstance = new PostprocessInstance();
            postInstance.setInstanceId(instanceId);
            postInstance.setPostState(PipelineFinal.RUN_HALT);
            String postInstanceId = postInstanceService.createPostInstance(postInstance);

            Tasks task = tasksService.findOnePostTaskOrTask(postprocess.getPostId());
            task.setTaskSort(postprocess.getTaskSort());
            fileAddress = fileAddress + "/" +  postInstanceId;
            String taskInstanceId = tasksExecService.createTaskExecInstance(task, postInstanceId, 3, fileAddress);
            task.setInstanceId(taskInstanceId);

            postInstance.setTaskInstanceId(taskInstanceId);
            task.setTaskSort(postprocess.getTaskSort());
            postprocess.setTask(task);
            postprocess.setValues(task);
            postprocess.setInstanceId(postInstanceId);
        }
        return postprocessList;
    }


    public String findPostInstanceId(String postId){
        return postIdOrPostInstanceId.get(postId);
    }

    public PostprocessInstance findPostInstance(String postInstanceId){
        return postInstanceIdOrPostInstance.get(postInstanceId);
    }


    private List<String> findPipelineOrPostInstanceCache(String pipelineId){
        List<String> list = pipelineIdOrPostInstanceId.get(pipelineId);
        if (Objects.isNull(list)){
            return new ArrayList<>();
        }
        return list;
    }
}
